/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Mirko Waechter <waechter at kit dot edu>
 * @copyright  2013 Mirko Waechter
 * @license    http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_UNITS_FORCETORQUE_SLICE_
#define _ARMARX_ROBOTAPI_UNITS_FORCETORQUE_SLICE_

#include <RobotAPI/interface/units/UnitInterface.ice>
#include <RobotAPI/interface/robotstate/RobotState.ice>
#include <RobotAPI/interface/robotstate/PoseBase.ice>

#include <Core/interface/core/UserException.ice>
#include <Core/interface/core/BasicTypes.ice>
#include <Core/interface/observers/VariantBase.ice>
#include <Core/interface/observers/ObserverInterface.ice>

module armarx
{

    interface TCPControlUnitInterface extends armarx::SensorActorUnitInterface, armarx::KinematicUnitListener
    {
        void setCycleTime(int milliseconds);
        void setTCPVelocity(string robotNodeSetName, string tcpNodeName, FramedVector3Base translationVelocity, FramedVector3Base orientationVelocityRPY);

    };


    dictionary<string, FramedPoseBase> FramedPoseBaseMap;
    interface TCPControlUnitListener
    {
        void reportTCPPose(FramedPoseBaseMap tcpPoses);
        void reportTCPVelocities(FramedVector3Map tcpTranslationVelocities, FramedVector3Map tcpOrienationVelocities);

    };

    interface TCPControlUnitObserverInterface extends ObserverInterface, TCPControlUnitListener
    {

    };

};

#endif
