#ifndef WEISSHAPTICSENSOR_H
#define WEISSHAPTICSENSOR_H

#include <Core/core/services/tasks/RunningTask.h>
#include <Core/core/system/ImportExportComponent.h>
#include "SerialInterface.h"
#include "TactileSensor.h"
#include <RobotAPI/interface/units/HapticUnit.h>
#include <Core/util/variants/eigen3/VariantObjectFactories.h>
#include "TextWriter.h"
#include <Core/observers/variant/TimestampVariant.h>
#include <Core/util/variants/eigen3/MatrixVariant.h>

namespace armarx
{
    class WeissHapticSensor : public Logging
    {
    public:
        WeissHapticSensor(std::string device);

        void connect();
        void disconnect();

        void setListenerPrx(HapticUnitListenerPrx listenerPrx);
        void startSampling();

    private:
        RunningTask<WeissHapticSensor>::pointer_type sensorTask;
        boost::shared_ptr<TextWriter> jsWriter;
        void frameAcquisitionTaskLoop();
        std::string device;
        std::string deviceFileName;
        boost::shared_ptr<SerialInterface> interface;
        boost::shared_ptr<TactileSensor> sensor;
        bool connected;
        string tag;
        tac_matrix_info_t mi;
        HapticUnitListenerPrx listenerPrx;
        void writeMatrixToJs(MatrixFloatPtr matrix, TimestampVariantPtr timestamp);
    };
}

#endif
