#include "ConditionCheckMagnitudeChecks.h"

namespace armarx {
    
    ConditionCheckMagnitudeLarger::ConditionCheckMagnitudeLarger()
    {
        setNumberParameters(1);
        addSupportedType(VariantType::Vector3, createParameterTypeList(1, VariantType::Float));
        addSupportedType(VariantType::FramedVector3, createParameterTypeList(1, VariantType::Float));
        addSupportedType(VariantType::LinkedVector3, createParameterTypeList(1, VariantType::Float));

    }

    ConditionCheck* ConditionCheckMagnitudeLarger::clone()
    {
        return new ConditionCheckMagnitudeLarger(*this);
    }

    bool ConditionCheckMagnitudeLarger::evaluate(const StringVariantMap &dataFields)
    {

        if(dataFields.size() != 1)
        {
            printf("Size of dataFields: %d\n", (int)dataFields.size());
            throw InvalidConditionException("Wrong number of datafields for condition equals ");
        }

        const Variant& value = dataFields.begin()->second;
        VariantTypeId type = value.getType();

        if(type == VariantType::Vector3)
        {
            Eigen::Vector3f vec = value.getClass<Vector3>()->toEigen();
            Eigen::Vector3f vec2 = getParameter(0).getClass<Vector3>()->toEigen();
            return (vec-vec2).norm() > getParameter(1).getFloat();
        }

        if(type == VariantType::FramedVector3)
        {
            FramedVector3Ptr fV1 = value.getClass<FramedVector3>();
            FramedVector3Ptr fV2 = getParameter(0).getClass<FramedVector3>();
            if(fV1->frame != fV2->frame)
            {
                ARMARX_WARNING_S << deactivateSpam(3) << "Cannot compare FramedVector3 with different reference frames, because no robot is here available. Use LinkedVector3 instead or provide in correct frames";
                return false;
            }

            Eigen::Vector3f vec = value.getClass<FramedVector3>()->toEigen();
            Eigen::Vector3f vec2 = getParameter(0).getClass<FramedVector3>()->toEigen();
            return (vec-vec2).norm() > getParameter(1).getFloat();
        }

        if(type == VariantType::LinkedVector3)
        {
            LinkedVector3Ptr lV1 = value.getClass<LinkedVector3>();
            LinkedVector3Ptr lV2 = getParameter(0).getClass<LinkedVector3>();
            if(lV1->frame != lV2->frame)
                lV1->changeFrame(lV2->frame);

            Eigen::Vector3f vec = value.getClass<LinkedVector3>()->toEigen();
            Eigen::Vector3f vec2 = getParameter(0).getClass<LinkedVector3>()->toEigen();
            return (vec-vec2).norm() > getParameter(1).getFloat();
        }

        return false;
    }


    
}
