#ifndef _ARMARX_ROBOTAPI_FORCETORQUEOBSERVER_H
#define _ARMARX_ROBOTAPI_FORCETORQUEOBSERVER_H

#include <RobotAPI/interface/units/ForceTorqueUnit.h>
#include <RobotAPI/libraries/robotstate/remote/ArmarPose.h>
#include <Core/observers/Observer.h>

namespace armarx
{

    class ForceTorqueObserverPropertyDefinitions:
            public ComponentPropertyDefinitions
    {
    public:
        ForceTorqueObserverPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineRequiredProperty<std::string>("ForceTorqueTopicName","Name of the ForceTorqueUnit Topic");
        }
    };

    /**
     * The ForceTorqueObserver monitors F/T values published by ForceTorqueUnit-implementations and offers condition checks on these values.
     * Available condition checks are: updated, larger, equals, smaller, magnitudelarger.
     */
    class ForceTorqueObserver :
            virtual public Observer,
            virtual public ForceTorqueUnitObserverInterface
    {
    public:
        ForceTorqueObserver();

        void setTopicName(std::string topicName);

        // framework hooks
        virtual std::string getDefaultName() const { return "ForceTorqueUnitObserver"; }
        void onInitObserver();
        void onConnectObserver();

        //void reportSensorValues(const std::string & type, const ::armarx::FramedVector3Map& newForces, const ::Ice::Current& = ::Ice::Current());
        //void reportSensorForceValues(const ::armarx::FramedVector3Map& newForces, const ::Ice::Current& = ::Ice::Current());
        //void reportSensorTorqueValues(const ::armarx::FramedVector3Map& newTorques, const ::Ice::Current& = ::Ice::Current());
        void reportSensorValues(const ::armarx::FramedVector3BasePtr &forces, const ::armarx::FramedVector3BasePtr &torques, const ::Ice::Current& = ::Ice::Current());
        void reportSensorForceValues(const ::armarx::FramedVector3BasePtr &forces, const ::Ice::Current& = ::Ice::Current());
        void reportSensorTorqueValues(const ::armarx::FramedVector3BasePtr &torques, const ::Ice::Current& = ::Ice::Current());


//        void reportForceWithOffset(const ::armarx::FramedVector3Map& newForces, const ::Ice::Current& = ::Ice::Current());
//        void reportTorqueRaw(const ::armarx::FramedVector3Map& newTorques, const ::Ice::Current& = ::Ice::Current());
//        void reportTorqueWithOffset(const ::armarx::FramedVector3Map& newTorques, const ::Ice::Current& = ::Ice::Current());

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual PropertyDefinitionsPtr createPropertyDefinitions();
    private:
        armarx::Mutex dataMutex;
        std::string topicName;

    };
}

#endif
