/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::units
 * @author     Peter Kaiser (peter dot kaiser at kit dot edu)
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */


#include "ForceTorqueUnitSimulation.h"

#include <RobotAPI/libraries/robotstate/remote/ArmarPose.h>

#include <boost/algorithm/string.hpp>

using namespace armarx;

void ForceTorqueUnitSimulation::onInitForceTorqueUnit()
{
    int intervalMs = getProperty<int>("IntervalMs").getValue();
    std::string sensorName = getProperty<std::string>("SensorName").getValue();

    //std::vector<std::string> sensorNamesList;
    //boost::split(sensorNamesList, sensorNames, boost::is_any_of(","));

    //for(std::vector<std::string>::iterator s = sensorNamesList.begin(); s != sensorNamesList.end(); s++)
    //{
    //    forces[*s] = new armarx::FramedVector3(Eigen::Vector3f(0, 0, 0), *s);
    //    torques[*s] = new armarx::FramedVector3(Eigen::Vector3f(0, 0, 0), *s);
   // }
    forces = new armarx::FramedVector3(Eigen::Vector3f(0, 0, 0), sensorName);
    torques = new armarx::FramedVector3(Eigen::Vector3f(0, 0, 0), sensorName);

    ARMARX_VERBOSE << "Starting ForceTorqueUnitSimulation with " << intervalMs << " ms interval";
    simulationTask = new PeriodicTask<ForceTorqueUnitSimulation>(this, &ForceTorqueUnitSimulation::simulationFunction, intervalMs, false, "ForceTorqueUnitSimulation");
}

void ForceTorqueUnitSimulation::onStartForceTorqueUnit()
{
    simulationTask->start();
}


void ForceTorqueUnitSimulation::onExitForceTorqueUnit()
{
    simulationTask->stop();
}


void ForceTorqueUnitSimulation::simulationFunction()
{
    listenerPrx->reportSensorForceValues(forces);
    listenerPrx->reportSensorTorqueValues(torques);

    //listenerPrx->reportSensorValues(forces);
    //listenerPrx->reportSensorValues(torques);
}

void ForceTorqueUnitSimulation::setOffset(const FramedVector3BasePtr &forceOffsets, const FramedVector3BasePtr &torqueOffsets, const Ice::Current &c)
{
    // Ignore
}

void ForceTorqueUnitSimulation::setToNull(const Ice::Current &c)
{
    // Ignore
}


PropertyDefinitionsPtr ForceTorqueUnitSimulation::createPropertyDefinitions()
{
    return PropertyDefinitionsPtr(new ForceTorqueUnitSimulationPropertyDefinitions(getConfigIdentifier()));
}

