/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXCore::units
 * @author     David Schiebener (schiebener at kit dot edu)
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_CORE_UNITS_HANDUNITSIMULATION_H
#define _ARMARX_CORE_UNITS_HANDUNITSIMULATION_H


#include "HandUnit.h"

namespace armarx
{
    /**
     * @class HandUnitSimulationPropertyDefinitions
     * @brief Defines all necessary properties for armarx::HandUnitSimulation
     * @ingroup SensorActorUnits
     */
    class HandUnitSimulationPropertyDefinitions:
            public ComponentPropertyDefinitions
    {
    public:
        HandUnitSimulationPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineRequiredProperty<std::string>("RobotFileName","VirtualRobot XML file in which the endeffector is is stored.");
            defineRequiredProperty<std::string>("EndeffectorName","Name of the endeffector as stored in the XML file (will publish values on EndeffectorName + 'State')");
        }
    };


    /**
     * @class HandUnitSimulation
     * @brief This class defines an interface for providing high level access to robot hands
     * @ingroup SensorActorUnits
     *
     * An instance of a HandUnitSimulation provides means to open, close, and preshape hands.
     * It uses the HandUnitListener Ice interface to report updates of its current state
     *
     */
    class HandUnitSimulation :
        virtual public HandUnit
    {
    public:
        // inherited from Component
        virtual std::string getDefaultName() const
        {
            return "HandUnitSimulation";
        }

        /**
         *
         */
        virtual void onInitHandUnit();
        /**
         *
         */
        virtual void onStartHandUnit();
        /**
         *
         */
        virtual void onExitHandUnit();

        /**
         * Send command to the hand to open all fingers.
         */
        virtual void open(const Ice::Current& c = ::Ice::Current());

        /**
         * Send command to the hand to close all fingers.
         */
        virtual void close(const Ice::Current& c = ::Ice::Current());

        /**
         * Send command to the hand to form a specific preshape position.
         */
        virtual void preshape(const std::string& preshapeName, const Ice::Current& c = ::Ice::Current());


    protected:
    };
}

#endif
