/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::
* @author     David Schiebener ( schiebener at kit dot edu)
* @date       2014
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef _ARMARX_HEADIKUNIT_H
#define _ARMARX_HEADIKUNIT_H


#include <Core/core/Component.h>
#include <Core/core/services/tasks/PeriodicTask.h>
#include <RobotAPI/libraries/robotstate/remote/ArmarPose.h>
#include <RobotAPI/libraries/robotstate/remote/RemoteRobot.h>

#include <RobotAPI/interface/units/HeadIKUnit.h>


namespace armarx
{


    class HeadIKUnitPropertyDefinitions:
            public ComponentPropertyDefinitions
    {
    public:
        HeadIKUnitPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineRequiredProperty<std::string>("KinematicUnitName","Name of the KinematicUnit Proxy");
            defineOptionalProperty<int>("CycleTime", 30, "Cycle time of the tcp control in ms");
        }
    };

    class HeadIKUnit : virtual public Component, virtual public HeadIKUnitInterface
    {
    public:
        HeadIKUnit();
        void onInitComponent();
        void onConnectComponent();
        void onDisconnectComponent();
        void onExitComponent();
        std::string getDefaultName() const{ return "HeadIKUnit"; }

        // HeadIKUnitInterface interface
        void setCycleTime(Ice::Int milliseconds, const Ice::Current &c = Ice::Current());
        void setHeadTarget(const std::string& robotNodeSetName, const FramedVector3BasePtr& targetPosition, const Ice::Current &c = Ice::Current());

        // UnitExecutionManagementInterface interface
        void init(const Ice::Current &c = Ice::Current());
        void start(const Ice::Current &c = Ice::Current());
        void stop(const Ice::Current &c = Ice::Current());
        UnitExecutionState getExecutionState(const Ice::Current &c = Ice::Current());

        // UnitResourceManagementInterface interface
        void request(const Ice::Current &c = Ice::Current());
        void release(const Ice::Current &c = Ice::Current());

        // PropertyUser interface
        PropertyDefinitionsPtr createPropertyDefinitions();


    private:
        void periodicExec();

        Mutex accessMutex;
        bool requested;
        int cycleTime;
        PeriodicTask<HeadIKUnit>::pointer_type execTask;

        RobotStateComponentInterfacePrx robotStateComponentPrx;
        KinematicUnitInterfacePrx kinematicUnitPrx;
        //SharedRobotInterfacePrx remoteRobotPrx;
        VirtualRobot::RobotPtr localRobot;

        std::string robotNodeSetName;
        FramedPositionPtr targetPosition;
        bool newTargetSet;
    };

}


#endif // _ARMARX_HEADIKUNIT_H
