/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::MotionControlGroup
 * @author     Mirko Waechter ( mirko dot waechter at kit dot edu )
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_XMLUSERCODE_RobotAPI_MotionControlGroup_MoveJoints_H
#define _ARMARX_XMLUSERCODE_RobotAPI_MotionControlGroup_MoveJoints_H

#include <Core/statechart/xmlstates/XMLState.h>
//#include <Core/statechart/StateUtilFunctions.h>

namespace armarx
{
    namespace MotionControlGroup
    {
        DEFINEEVENT(EvJointTargetReached)
        DEFINEEVENT(EvTimeout)
        class MoveJoints :
            virtual public XMLStateTemplate < MoveJoints > ,
        public XMLStateFactoryBase
        {
        public:
            MoveJoints(XMLStateConstructorParams stateData);

            // inherited from StateBase
            void onEnter();
            void onBreak();
            void onExit();

            // static functions for AbstractFactory Method
            static std::string GetName();
            static XMLStateFactoryBasePtr CreateInstance(XMLStateConstructorParams stateData);
            static SubClassRegistry Registry;

            ConditionIdentifier targetReachedCondition;
            ActionEventIdentifier timeoutEvent;

            // DO NOT INSERT ANY CLASS MEMBERS,
            // use stateparameters instead,
            // if classmember are neccessary nonetheless, reset them in onEnter
        };
    }
}

#endif
