/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::MotionControlGroup
 * @author     Manfred Kroehnert ( Manfred dot Kroehnert at kit dot edu )
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#include "StopRobot.h"

#include <RobotAPI/libraries/core/RobotStatechartContext.h>

using namespace armarx;
using namespace MotionControlGroup;

// DO NOT EDIT NEXT LINE
StopRobot::SubClassRegistry StopRobot::Registry(StopRobot::GetName(), &StopRobot::CreateInstance);



StopRobot::StopRobot(XMLStateConstructorParams stateData) :
    XMLStateTemplate < StopRobot > (stateData)
{
}

void StopRobot::onEnter()
{
    ARMARX_LOG << "entering MotionControl::StopRobot" << flush;

    RobotStatechartContext* context = getContext<RobotStatechartContext>();

    armarx::NameList allNodes = context->robotStateComponent->getRobotSnapshot("StopRobotTime")->getRobotNodes();

    NameValueMap jointVelocities;
    for (const std::string& nodeName : allNodes)
    {
        jointVelocities[nodeName] = 0.0f;
    }

    context->kinematicUnitPrx->begin_setJointVelocities(jointVelocities);

    sendEvent<EvSuccess>();
}

void StopRobot::run()
{
    // put your user code for the execution-phase here
    // runs in seperate thread, thus can do complex operations
    // should check constantly whether isRunningTaskStopped() returns true


    while (!isRunningTaskStopped()) // stop run function if returning true
    {
        // do your calculations
    }

}

void StopRobot::onBreak()
{
    // put your user code for the breaking point here
    // execution time should be short (<100ms)
}

void StopRobot::onExit()
{
    // put your user code for the exit point here
    // execution time should be short (<100ms)

}

// DO NOT EDIT NEXT FUNCTION
std::string StopRobot::GetName()
{
    return "StopRobot";
}

// DO NOT EDIT NEXT FUNCTION
XMLStateFactoryBasePtr StopRobot::CreateInstance(XMLStateConstructorParams stateData)
{
    return XMLStateFactoryBasePtr(new StopRobot(stateData));
}

