/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    RobotAPI::PlaceObject
* @author     Nikolaus Vahrenkamp
* @date       2014
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef ARMARX_COMPONENT_PLACE_OBJECT_H
#define ARMARX_COMPONENT_PLACE_OBJECT_H

#include <Core/statechart/Statechart.h>


namespace armarx
{
    // ****************************************************************
    // Events
    // ****************************************************************

    DEFINEEVENT(EvPlaceObjectTimeout)
    DEFINEEVENT(EvPlaceObjectCollision)


    // ****************************************************************
    // Definition of StatechartPlaceObject
    // ****************************************************************
    struct StatechartPlaceObject :
        StateTemplate<StatechartPlaceObject>
    {
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();
        StateUtility::ActionEventIdentifier condPlaceObjectTimeout;
        ConditionIdentifier condPlaceObjectCollision;
    };



    // ****************************************************************
    // Definition of states
    // ****************************************************************
    /**
     * StatePlaceMoveDown: Move down until a surface is hit
     */

    struct StateMoveDown :
        StateTemplate<StateMoveDown>
    {
        void defineParameters();
        void onEnter();
        void onExit();
    };


    /**
     * StateCheckForces: Check if a surface was hit -> forces must increase
     */
    /*struct StateCheckForces :
        StateTemplate<StateCheckForces>
    {
        void defineParameters();
        void onEnter();
        void onExit();
    };*/

}

#endif
